@@ -32,8 +32,11 @@ import os
 import shutil
 import sys
 
+from sphinx_astropy.conf.v1 import intersphinx_mapping, default_role
+
 import sphinx_rtd_theme
 
+
 # Based on http://read-the-docs.readthedocs.io/en/latest/faq.html#i-get-import-errors-on-libraries-that-depend-on-c-modules
 #
 from unittest.mock import MagicMock as BaseMock
@@ -146,11 +149,16 @@ extensions = [
     # of warning messages (about missing links) that I don't have time
     # to investigate.
     'sphinx.ext.napoleon',
-    # 'numpydoc.numpydoc'
+    # 'numpydoc.numpydoc',
+    'sphinx.ext.intersphinx',
+    'sphinx_astropy.ext.intersphinx_toggle',
     'sphinx_astropy.ext.edit_on_github',
     # notebooks
     'nbsphinx'
 ]
+# Imported from sphinx_astropy so we don't have to maintain the list
+# of servers
+# intersphinx_mapping = {'python': ('https://docs.python.org/3', None)}
 
 # notebook support
 # - for now never execute a notebook
@@ -273,8 +281,8 @@ source_suffix = '.rst'
 #
 # source_encoding = 'utf-8-sig'
 
-# The master toctree document.
-master_doc = 'index'
+# The main toctree document.
+main_doc = 'index'
 
 # General information about the project.
 project = 'Sherpa'
@@ -313,8 +321,11 @@ exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']
 
 # The reST default role (used for this markup: `text`) to use for all
 # documents.
-#
-# default_role = None
+# Imported from sphinx_astropy
+# default_role = 'obj'
+
+# Setting from sphinx_astropy for numpydoc xref settings
+numpydoc_xref_param_type = True
 
 # If true, '()' will be appended to :func: etc. cross-reference text.
 #
@@ -343,7 +354,7 @@ pygments_style = 'sphinx'
 todo_include_todos = False
 
 # Graphviz based on values from AstroPy - see
-# https://github.com/astropy/sphinx-astropy/blob/master/sphinx_astropy/conf/v1.py
+# https://github.com/astropy/sphinx-astropy/blob/main/sphinx_astropy/conf/v1.py
 #
 graphviz_output_format = "svg"
 
@@ -508,7 +519,7 @@ latex_elements = {
 # (source start file, target name, title,
 #  author, documentclass [howto, manual, or own class]).
 latex_documents = [
-    (master_doc, 'Sherpa.tex', 'Sherpa Documentation',
+    (main_doc, 'Sherpa.tex', 'Sherpa Documentation',
      'Chandra X-ray Center', 'manual'),
 ]
 
@@ -550,7 +561,7 @@ latex_documents = [
 # One entry per manual page. List of tuples
 # (source start file, name, description, authors, manual section).
 man_pages = [
-    (master_doc, 'sherpa', 'Sherpa Documentation',
+    (main_doc, 'sherpa', 'Sherpa Documentation',
      [author], 1)
 ]
 
@@ -565,7 +576,7 @@ man_pages = [
 # (source start file, target name, title, author,
 #  dir menu entry, description, category)
 texinfo_documents = [
-    (master_doc, 'Sherpa', 'Sherpa Documentation',
+    (main_doc, 'Sherpa', 'Sherpa Documentation',
      author, 'Sherpa',
      'Sherpa is a modeling and fitting environment for Python.',
      'Miscellaneous'),
