@@ -36,6 +36,7 @@ from sherpa.astro.utils import bounds_check
 from sherpa.utils.err import PlotErr, IOErr
 from sherpa.utils import parse_expr, dataspace1d, histogram1d, filter_bins, \
     sao_fcmp
+from sherpa.astro import hc
 
 warning = logging.getLogger(__name__).warning
 
@@ -53,7 +54,7 @@ __all__ = ('DataPHAPlot', 'ModelPHAHistogram', 'ModelHistogram',
 _tol = np.finfo(np.float32).eps
 
 
-def _check_hist_bins(plot):
+def _check_hist_bins(xlo, xhi):
     """Ensure lo/hi edges that are "close" are merged.
 
     Ensure that "close-enough" bin edges use the same value.  We do
@@ -64,25 +65,49 @@ def _check_hist_bins(plot):
 
     Parameters
     ----------
-    plot
-        The plot structure, which must have xlo and xhi attributes.
+    xlo, xhi : array
+        Lower and upper bin boundaries. Typically, ``xlo`` will contain the
+        lower boundary and ``xhi`` the upper boundary, but this function can
+        deal with situations where that is reversed. Both arrays have to be
+        monotonically increasing or decreasing.
+
+    Returns
+    -------
+    xlo, xhi : array
+        xlo and xhi with values that were very close (within numerical
+        tolerance) before changed such that they now match exactly.
 
     Notes
     -----
     Note that this holds even when plotting wavelength values, who
     have xlo/xhi in decreasing order, since the lo/hi values still
     hold.
-
     """
+    if len(xlo) != len(xhi):
+        # Not a Sherpa specific error, because this is more for developers.
+        raise ValueError('Input arrays must have same length.')
+    # Nothing to compare if input arrays are empty.
+    if len(xlo) == 0:
+        return xlo, xhi
 
     # Technically idx should be 0 or 1, with no -1 values. We
     # do not enforce this. What we do is to take all bins that
     # appear similar (sao_fcmp==0) and set the xlo[i+1] bin
     # to the xhi[i] value.
     #
-    equal = sao_fcmp(plot.xlo[1:], plot.xhi[:-1], _tol)
+    # Deal with xhi <-> xlo switches. Those can occor when converting
+    # from energy to wavelength.
+    # Deal with reversed order. Can happen when converting from energy
+    # to wavelength, or if input PHA is not ordered in increasing energy.
+    # But is both are happening at the same time, need to switch twice, which
+    # is a no-op. So, we get to use the elusive Python XOR operator.
+    if (xlo[0] > xhi[0]) ^ (xhi[0] > xhi[-1]):
+        xlo, xhi = xhi, xlo
+    equal = sao_fcmp(xlo[1:], xhi[:-1], _tol)
     idx, = np.where(equal == 0)
-    plot.xlo[idx + 1] = plot.xhi[idx]
+    xlo[idx + 1] = xhi[idx]
+
+    return xlo, xhi
 
 
 def to_latex(txt):
@@ -134,10 +159,10 @@ class DataPHAPlot(sherpa.plot.DataHistogramPlot):
         self.xlo = data.apply_filter(elo, data._min)
         self.xhi = data.apply_filter(ehi, data._max)
         if data.units == 'wavelength':
-            self.xlo = data._hc / self.xlo
-            self.xhi = data._hc / self.xhi
+            self.xlo = hc / self.xlo
+            self.xhi = hc / self.xhi
 
-        _check_hist_bins(self)
+        self.xlo, self.xhi = _check_hist_bins(self.xlo, self.xhi)
 
 
 class ModelPHAHistogram(HistogramPlot):
@@ -170,41 +195,66 @@ class ModelPHAHistogram(HistogramPlot):
         self.xlo = data.apply_filter(elo, data._min)
         self.xhi = data.apply_filter(ehi, data._max)
         if data.units == 'wavelength':
-            self.xlo = data._hc / self.xlo
-            self.xhi = data._hc / self.xhi
+            self.xlo = hc / self.xlo
+            self.xhi = hc / self.xhi
 
-        _check_hist_bins(self)
+        self.xlo, self.xhi = _check_hist_bins(self.xlo, self.xhi)
 
 
 class ModelHistogram(ModelPHAHistogram):
     """Plot a model for a PHA dataset with no grouping.
 
-    The model is drawn at the native resolution of
-    the instrument response, or ungrouped channels.
-    The model is filtered to the minimum and maximum
-    of the PHA dataset but any ignored ranges within
-    this are ignored.
+    The model is drawn at the native resolution of the instrument
+    response, or ungrouped channels.
+
     """
 
     def prepare(self, data, model, stat=None):
 
-        old_filter = parse_expr(data.get_filter())
-        old_group = data.grouped
+        # We could fit this into a single try/finally group but
+        # it makes it harder to see what is going on so split
+        # it out into separate plot types:
+        #   - ungrouped
+        #   - all data has been masked out (we let prepare
+        #     throw any errors or plot no data)
+        #   - grouped but no filter (or the filter doesn't
+        #     remove any points).
+        #   - grouped and filtered
+        #
+        if not data.grouped or data.mask is False or \
+           (data.mask is not True and not data.mask.any()):
+            super().prepare(data, model, stat=stat)
+            return
+
+        # At this point mask can be True or an array.
+        #
+        if data.mask is True or data.mask.all():
+            try:
+                data.ungroup()
+                super().prepare(data, model, stat=stat)
+
+            finally:
+                data.group()
+
+            return
+
+        # We need to convert the filter expression from grouped
+        # to ungrouped, apply it, create the plot, then restore
+        # the old expression. Note that we can just copy over
+        # the original mask value to restore the old filter.
+        #
+        old_mask = data.mask.copy()
         new_filter = parse_expr(data.get_filter(group=False))
         try:
-            if old_group:
-                data.ungroup()
-                for interval in new_filter:
-                    data.notice(*interval)
+            data.ungroup()
+            for interval in new_filter:
+                data.notice(*interval)
 
             super().prepare(data, model, stat=stat)
 
         finally:
-            if old_group:
-                data.ignore()
-                data.group()
-                for interval in old_filter:
-                    data.notice(*interval)
+            data.group()
+            data.mask = old_mask
 
 
 class SourcePlot(HistogramPlot):
@@ -245,9 +295,17 @@ class SourcePlot(HistogramPlot):
             self.units = "energy"
 
         self.xlabel = data.get_xlabel()
-        self.title = 'Source Model of %s' % data.name
+        self.title = f'Source Model of {data.name}'
         self.xlo, self.xhi = data._get_indep(filter=False)
+
+        # Why do we not apply the mask at the end of prepare?
+        #
         self.mask = filter_bins((lo,), (hi,), (self.xlo,))
+
+        # The source model is assumed to not contain an instrument model,
+        # and so it evaluates the expected number of photons/cm^2/s in
+        # each bin (or it can be thought of as a 1 second exposure).
+        #
         self.y = src(self.xlo, self.xhi)
         prefix_quant = 'E'
         quant = 'keV'
@@ -265,23 +323,21 @@ class SourcePlot(HistogramPlot):
 
         sqr = to_latex('^2')
 
-        self.xlabel = '%s (%s)' % (self.units.capitalize(), quant)
-        self.ylabel = '%s  Photons/sec/cm' + sqr + '%s'
+        self.xlabel = f'{self.units.capitalize()} ({quant})'
+        self.ylabel = f'%s  Photons/sec/cm{sqr}%s'
 
         if data.plot_fac == 0:
             self.y /= xmid
-            self.ylabel = self.ylabel % ('f(%s)' % prefix_quant,
-                                         '/%s ' % quant)
+            self.ylabel = self.ylabel % (f'f({prefix_quant})',
+                                         f'/{quant} ')
 
         elif data.plot_fac == 1:
-            self.ylabel = self.ylabel % ('%s f(%s)' % (prefix_quant,
-                                                       prefix_quant), '')
+            self.ylabel = self.ylabel % (f'{prefix_quant} f({prefix_quant})', '')
 
         elif data.plot_fac == 2:
             self.y *= xmid
-            self.ylabel = self.ylabel % ('%s%s f(%s)' % (prefix_quant, sqr,
-                                                         prefix_quant),
-                                         ' %s ' % quant)
+            self.ylabel = self.ylabel % (f'{prefix_quant}{sqr} f({prefix_quant})',
+                                         f' {quant} ')
         else:
             raise PlotErr('plotfac', 'Source', data.plot_fac)
 
@@ -389,8 +445,8 @@ class ARFPlot(HistogramPlot):
                 raise PlotErr('notpha', data.name)
             if data.units == "wavelength":
                 self.xlabel = 'Wavelength (Angstrom)'
-                self.xlo = data._hc / self.xlo
-                self.xhi = data._hc / self.xhi
+                self.xlo = hc / self.xlo
+                self.xhi = hc / self.xhi
 
 
 class BkgDataPlot(DataPHAPlot):
@@ -509,8 +565,8 @@ class OrderPlot(ModelHistogram):
                 self.colors.append(top_color)
                 top_color = hex(int(top_color, 16) - jump)
 
-        if not self.use_default_colors and len(colors) != len(orders):
-            raise PlotErr('ordercolors', len(orders), len(colors))
+        if not self.use_default_colors and len(self.colors) != len(self.orders):
+            raise PlotErr('ordercolors', len(self.orders), len(self.colors))
 
         old_filter = parse_expr(data.get_filter())
         old_group = data.grouped
@@ -532,14 +588,16 @@ class OrderPlot(ModelHistogram):
                 xlo = data.apply_filter(elo, data._min)
                 xhi = data.apply_filter(ehi, data._max)
                 if data.units == 'wavelength':
-                    xlo = data._hc / xlo
-                    xhi = data._hc / xhi
+                    xlo = hc / xlo
+                    xhi = hc / xhi
             else:
                 xhi = xlo + 1.
 
             for order in self.orders:
                 self.xlo.append(xlo)
                 self.xhi.append(xhi)
+                # QUS: why check that response_ids > 2 and not 1 here?
+                #
                 if len(data.response_ids) > 2:
                     if order < 1 or order > len(model.rhs.orders):
                         raise PlotErr('notorder', order)
@@ -610,9 +668,9 @@ class FluxHistogram(ModelHistogram):
             flux = array2string(asarray(self.flux), separator=',',
                                 precision=4, suppress_small=False)
 
-        return '\n'.join(['modelvals = {}'.format(vals),
-                          'clipped = {}'.format(clip),
-                          'flux = {}'.format(flux),
+        return '\n'.join([f'modelvals = {vals}',
+                          f'clipped = {clip}',
+                          f'flux = {flux}',
                           ModelHistogram.__str__(self)])
 
     def prepare(self, fluxes, bins):
@@ -648,8 +706,7 @@ class EnergyFluxHistogram(FluxHistogram):
     def __init__(self):
         FluxHistogram.__init__(self)
         self.title = "Energy flux distribution"
-        self.xlabel = "Energy flux (ergs cm{} sec{})".format(
-            to_latex('^{-2}'), to_latex('^{-1}'))
+        self.xlabel = f"Energy flux (ergs cm{to_latex('^{-2}')} sec{to_latex('^{-1}')})"
         self.ylabel = "Frequency"
 
 
@@ -659,6 +716,5 @@ class PhotonFluxHistogram(FluxHistogram):
     def __init__(self):
         FluxHistogram.__init__(self)
         self.title = "Photon flux distribution"
-        self.xlabel = "Photon flux (Photons cm{} sec{})".format(
-            to_latex('^{-2}'), to_latex('^{-1}'))
+        self.xlabel = f"Photon flux (Photons cm{to_latex('^{-2}')} sec{to_latex('^{-1}')})"
         self.ylabel = "Frequency"
