@@ -1,6 +1,6 @@
 #
 #  Copyright (C) 2012, 2015, 2016, 2017, 2018, 2019, 2020, 2021
-#     Smithsonian Astrophysical Observatory
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -33,7 +33,7 @@ from sherpa.astro.data import DataPHA
 from sherpa.astro.instrument import RMFModelPHA
 from sherpa.astro import ui
 from sherpa.data import Data1D
-from sherpa.utils.err import ArgumentErr, DataErr, IdentifierErr, StatErr
+from sherpa.utils.err import ArgumentErr, DataErr, IdentifierErr, IOErr, StatErr
 from sherpa.utils.logging import SherpaVerbosity
 from sherpa.utils.testing import requires_data, requires_fits, \
     requires_group, requires_xspec
@@ -413,13 +413,15 @@ def test_bug38_filtering_grouping(make_data_path):
     pha.notice(1, 6)
     pha.ignore(3, 4)
 
-    assert pha.get_filter(group=True, format='%.4f') == '1.0147:2.7886,4.1391:6.2342'
-    assert pha.get_filter(group=False, format='%.4f') == '1.0001:2.8543,4.0369:6.5627'
+    expected = '0.9928:2.8616,4.0296:6.5700'
+    assert pha.get_filter(group=True, format='%.4f') == expected
+    assert pha.get_filter(group=False, format='%.4f') == expected
 
     pha.group_width(40)
 
-    assert pha.get_filter(group=True, format='%.4f') == '0.8760:2.6280,3.7960:6.7160'
-    assert pha.get_filter(group=False, format='%.4f') == '0.5913:2.9127,3.5113:7.0007'
+    expected = '0.5840:2.9200,3.5040:7.0080'
+    assert pha.get_filter(group=True, format='%.4f') == expected
+    assert pha.get_filter(group=False, format='%.4f') == expected
 
     assert pha.mask.size == 26
     assert pha.mask.sum() == 10
@@ -714,6 +716,21 @@ def test_save_arrays_ASCII(colnames):
     save_arrays(colnames=colnames, fits=False, read_func=read_func)
 
 
+
+@requires_fits
+@pytest.mark.parametrize("ascii_type", [False, True])
+def test_save_arrays_colmismatch_errs(ascii_type):
+    with pytest.raises(IOErr) as exc:
+        a = numpy.asarray([1, 3, 5])
+        b = numpy.asarray([4, 6, 8])
+        c = a*b
+        fields = ["odd", "even"]
+        ui.save_arrays("bogus_tempfile_name", [a, b, c], fields=fields,
+                   ascii=ascii_type, clobber=True)
+    assert 'Expected 3 columns but found 2' in str(exc.value)
+
+
+
 @requires_fits
 @requires_data
 def testWrite(make_data_path):
@@ -1115,7 +1132,7 @@ def test_grouped_pha_all_bad_channel(clean_astro_ui):
 @pytest.mark.parametrize("chantype,exp_counts,exp_xlo,exp_xhi,lo1,hi1,lo2,hi2",
                          [("channel", 0.8, 1.0, 6, 0, 7, 2, 6),
                           ("energy", 8.0, 0.1, 0.6, 0.05, 1.0, 0.2, 0.8),
-                          ("wave", 0.03871461, 123.9841874, 20.66403123, 20, 90, 30, 85)])
+                          ("wave", 0.03871461, 20.66403123, 123.9841874, 20, 90, 30, 85)])
 def test_grouped_pha_all_bad_response(arf, rmf, chantype, exp_counts, exp_xlo, exp_xhi, lo1, hi1, lo2, hi2, clean_astro_ui):
     """Helpdesk ticket: low-count data had no valid bins after grouping #790
 
@@ -1196,7 +1213,7 @@ def test_grouped_pha_all_bad_response_bg_warning(elo, ehi, nbins, bkg_id,
     name, lvl, msg = caplog.record_tuples[0]
     assert name == 'sherpa.astro.data'
     assert lvl == logging.INFO
-    assert msg.startswith('Skipping dataset /')
+    assert msg.startswith('Skipping dataset ')
     assert msg.endswith('/3c273_bg.pi: mask excludes all data')
 
 
