@@ -35,6 +35,7 @@ from sherpa.utils.testing import requires_data, requires_fits, \
 from sherpa.utils.err import ArgumentErr, ArgumentTypeErr, FitErr, \
     IdentifierErr, IOErr, ModelErr
 import sherpa.astro.utils
+from sherpa.astro import hc, charge_e
 
 
 def fail(*arg):
@@ -110,13 +111,13 @@ def test_calc_flux_pha_bin_edges(clean_astro_ui):
     faked data that is made to make the behavior "obvious".
     """
 
-    chans = np.arange(1, 11, 1, dtype=np.int)
-    counts = np.zeros(chans.size, dtype=np.int)
+    chans = np.arange(1, 11, 1, dtype=int)
+    counts = np.zeros(chans.size, dtype=int)
 
     # "perfect" response
     energies = np.arange(1, 12, 1)
     elo, ehi = energies[:-1], energies[1:]
-    flat = np.ones(chans.size, dtype=np.int)
+    flat = np.ones(chans.size, dtype=int)
 
     d = ui.DataPHA('example', chans, counts)
     arf = ui.create_arf(elo, ehi, flat)
@@ -137,8 +138,6 @@ def test_calc_flux_pha_bin_edges(clean_astro_ui):
     pflux = ui.calc_photon_flux(2.6, 7.8)
     eflux = ui.calc_energy_flux(2.6, 7.8)
 
-    enscale = sherpa.astro.utils._charge_e
-
     # Left in as notes:
     #
     # This are the true values. Given that the edge bins (should be)
@@ -163,7 +162,7 @@ def test_calc_flux_pha_bin_edges(clean_astro_ui):
     scale = np.asarray([0.0, 0.4, 1.0, 1.0, 1.0, 1.0, 0.8, 0, 0.0, 0.0])
     expected_pflux = (ymdl * scale).sum()
 
-    emid = enscale * (elo + ehi) / 2
+    emid = charge_e * (elo + ehi) / 2
     expected_eflux = (emid * ymdl * scale).sum()
 
     assert pflux == pytest.approx(expected_pflux)
@@ -181,13 +180,13 @@ def test_calc_flux_pha_density_bin_edges(clean_astro_ui):
     faked data that is made to make the behavior "obvious".
     """
 
-    chans = np.arange(1, 11, 1, dtype=np.int)
-    counts = np.zeros(chans.size, dtype=np.int)
+    chans = np.arange(1, 11, 1, dtype=int)
+    counts = np.zeros(chans.size, dtype=int)
 
     # "perfect" response
     energies = np.arange(1, 12, 1)
     elo, ehi = energies[:-1], energies[1:]
-    flat = np.ones(chans.size, dtype=np.int)
+    flat = np.ones(chans.size, dtype=int)
 
     d = ui.DataPHA('example', chans, counts)
     arf = ui.create_arf(elo, ehi, flat)
@@ -208,15 +207,13 @@ def test_calc_flux_pha_density_bin_edges(clean_astro_ui):
     pdens = ui.calc_photon_flux(2.6)
     edens = ui.calc_energy_flux(2.6)
 
-    enscale = sherpa.astro.utils._charge_e
-
     # Evaluate the model over the bin 2-3 keV; since the grid
     # has a width of 1 keV we do not need to divide by the bin
     # width when calculating the density.
     #
     ymdl = pl([2], [3])
     expected_pdens = ymdl.sum()
-    expected_edens = enscale * 2.5 * expected_pdens
+    expected_edens = charge_e * 2.5 * expected_pdens
 
     # Prior to fixing #619, Sherpa returns 0 for both densities
     #
@@ -323,7 +320,7 @@ def test_calc_flux_pha(id, lo, hi, make_data_path, clean_astro_ui):
     assert eflux == pytest.approx(eflux_exp, rel=1e-4)
 
 
-# The lo/hi range which match in the different settings; using _hc
+# The lo/hi range which match in the different settings; using hc
 # to convert from keV to Angstroms is a bit low-level.
 #
 # The energy-to-channel conversion was done by filtering in energy
@@ -335,7 +332,7 @@ def test_calc_flux_pha(id, lo, hi, make_data_path, clean_astro_ui):
                          [(None, None, 'wave', None, None),
                           (None, None, 'channel', None, None),
                           (0.5, 7.0, 'wave',
-                           ui.DataPHA._hc / 7.0, ui.DataPHA._hc / 0.5),
+                           hc / 7.0, hc / 0.5),
                           fail(0.5, 7.0, 'channel', 35, 480)  # issue 619 see also 308
                          ])
 def test_calc_flux_pha_analysis(elo, ehi, setting, lo, hi, make_data_path, clean_astro_ui):
@@ -2029,10 +2026,10 @@ def test_sample_flux_751_752(idval, make_data_path, clean_astro_ui,
     assert flsig > fluxes.min()
     assert fusig < fluxes.max()
 
-    # The clip values are not set (since these are set to the hard limits)
+    # The clip values are set.
     #
-    clips = vals[:, -2]
-    assert (clips == 0).all()
+    clipped = vals[:, -2] != 0
+    assert clipped.sum() == 20
 
     # All the statistic values should be set (> 0). This wasn't until #751
     # was addressed.
@@ -2040,6 +2037,24 @@ def test_sample_flux_751_752(idval, make_data_path, clean_astro_ui,
     stats = vals[:, -1]
     assert (stats > 0).all()
 
+    # We assume that the stats for the clipped values are worse
+    # than for the unclipped values. Unfortunately they are not guaranteed
+    # to not overlap, so all we can do is ensure some simple checks,
+    # such as the minimum values are different.
+    #
+    assert stats[clipped].min() >= stats[~clipped].min()
+
+    # check that the calculated median is the same as the median
+    # we can calculate with the unclipped data. This is for flux1 not
+    # flux2 (although in this test they are the same).
+    #
+    fcalc = np.median(fluxes[~clipped])
+    assert fcalc == pytest.approx(fmed)
+
+    # regression test
+    assert stats[clipped][0] == pytest.approx(51.56646084)
+    assert stats[~clipped][-1] == pytest.approx(22.10377607)
+
 
 @requires_xspec
 @requires_data
@@ -2093,17 +2108,12 @@ def test_sample_flux_457(make_data_path, clean_astro_ui,
     assert flux1.shape == (3, )
     assert vals.shape == (niter + 1, 6)
 
-    # How many clipped values are there?
+    # There are 17 "clipped" rows.
     #
-    clipped = vals[:, -2] == 1
-    nhs = vals[:, 1]
-    assert (nhs[~clipped] > 0).all()
-    assert (nhs[clipped] == 0).all()
+    clipped = vals[:, -2] != 0
     assert clipped.sum() == 17
 
-    # Although there are 17 clipped values, they all have a non-zero statistic
-    # as the values have been clipped to the soft limits. So in this case
-    # issue #751 isn't an issue.
+    # A check for issue #751.
     #
     stats = vals[:, -1]
     assert (stats > 0).sum() == (niter + 1)
@@ -2246,7 +2256,8 @@ def test_sample_flux_errors(make_data_path, clean_astro_ui,
     # Ideally the best-fit location is the best fit!
     assert stats.min() >= stat0
 
-    # Check the clip column too
+    # Check the clip column too. This lists the soft-clipped rows,
+    # which for this case is ?
     #
     clips = res[2][:, 3]
     assert (clips == 0).all()
@@ -2480,3 +2491,88 @@ def test_sample_flux_pha_bkg(idval, make_data_path, clean_astro_ui,
     # fluxes.
     #
     assert (bvals[:, 1:] == vals[:, 1:]).all()
+
+
+@requires_data
+@requires_fits
+@pytest.mark.parametrize("idval", [1, 2])
+def test_sample_flux_pha_full_model(idval, make_data_path, clean_astro_ui,
+                                    hide_logging, reset_seed, caplog):
+    """When using set_full_model we error out. Is it a nice error?"""
+
+    np.random.seed(9783)
+
+    niter = 100
+
+    ui.load_pha(idval, make_data_path('3c273.pi'))
+    ui.subtract(idval)
+    ui.ignore(None, 1)
+    ui.ignore(7, None)
+
+    rsp = ui.get_response(idval)
+
+    ui.set_full_model(idval, rsp(ui.const1d.scal * ui.powlaw1d.p1))
+
+    p1 = ui.get_model_component('p1')
+    scal = ui.get_model_component('scal')
+    scal.c0 = 0.8
+    scal.integrate = False
+    scal.c0.frozen = True
+
+    ui.fit(idval)
+    ui.covar(idval)
+
+    scal = ui.get_covar_results().parmaxes
+
+    with pytest.raises(IdentifierErr) as ie:
+        ui.sample_flux(modelcomponent=p1,
+                       id=idval, lo=1, hi=5, num=niter,
+                       correlated=False, scales=scal)
+
+    assert str(ie.value) == 'Please use calc_energy_flux as set_full_model was used'
+
+
+@requires_data
+@requires_fits
+@pytest.mark.parametrize("idval", [1, 2])
+def test_sample_flux_pha_bkg_full_model(idval, make_data_path, clean_astro_ui,
+                                        hide_logging, reset_seed, caplog):
+    """When using set_bkg_full_model we error out. Is it a nice error?"""
+
+    np.random.seed(9783)
+
+    niter = 100
+
+    ui.load_pha(idval, make_data_path('3c273.pi'))
+    ui.subtract(idval)
+    ui.ignore(None, 1)
+    ui.ignore(7, None)
+
+    # I don't think we can get covar for just the background data so we have
+    # had to fit the whole dataset.
+    #
+    brsp = ui.get_response(idval, bkg_id=1)
+    ui.set_bkg_full_model(idval, brsp(ui.powlaw1d.bpl))
+
+    bpl = ui.get_model_component('bpl')
+    bpl.gamma = 0.54
+    bpl.ampl = 6.4e-6
+
+    ui.fit_bkg(idval)
+
+    bscale = ui.get_bkg_scale(idval)
+
+    rsp = ui.get_response(idval)
+    ui.set_full_model(idval, rsp(ui.powlaw1d.pl) + bscale * rsp(bpl))
+
+    ui.fit(idval)
+    ui.covar(idval) # , bkg_id=1)
+
+    scal = ui.get_covar_results().parmaxes
+
+    with pytest.raises(IdentifierErr) as ie:
+        ui.sample_flux(modelcomponent=bpl, bkg_id=1,
+                       id=idval, lo=1, hi=5, num=niter,
+                       correlated=False, scales=scal)
+
+    assert str(ie.value) == 'Please use calc_energy_flux as set_bkg_full_model was used'
