@@ -1,5 +1,5 @@
-//  Copyright (C) 2007, 2015-2018, 2019, 2020
-//      Smithsonian Astrophysical Observatory
+//  Copyright (C) 2007, 2015-2018, 2019, 2020, 2021
+//  Smithsonian Astrophysical Observatory
 //
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -100,16 +100,22 @@ int _sherpa_init_xspec_library();
 // gammap
 // gammq
 //
-#include "xsFortran.h"
-
-// TODO: is this defined in an XSPEC header file?
-#define ABUND_SIZE (30) // number of elements in Solar Abundance table
-
-// C_<model> are declared here; the other models are defined in
+// TODO:: switch to C++ FuntionUtility interface rather than use xsFortran.h
+//
+// funcWrappers: C_<model> are declared here; the other models are defined in
 // functionMap.h but that requires using the XSPEC build location
 // rather than install location.
 //
+#ifdef XSPEC_12_12_0
+#include "XSFunctions/Utilities/xsFortran.h"
+#include "XSFunctions/funcWrappers.h"
+#else
+#include "xsFortran.h"
 #include "funcWrappers.h"
+#endif
+
+// TODO: is this defined in an XSPEC header file?
+#define ABUND_SIZE (30) // number of elements in Solar Abundance table
 
 extern "C" {
 
@@ -354,6 +360,14 @@ void kyconv_(float* ear, int* ne, float* param, int* ifl, float* photar, float*
 void thcompf_(float* ear, int* ne, float* param, int* ifl, float* photar, float* photer);
 #endif
 
+// XSPEC 12.12.0 changes
+#ifdef XSPEC_12_12_0
+// Note: have dropped the leading 'c_' for this model to follow xsgrbcomp
+void xsgrbjet(const double* energy, int nFlux, const double* params, int spectrumNumber, double* flux, double* fluxError, const char* initStr);
+
+void zxipab_(float* ear, int* ne, float* param, int* ifl, float* photar, float* photer);
+#endif
+
 }
 
 // This routine could be called when the module is being initialized,
@@ -362,7 +376,11 @@ void thcompf_(float* ear, int* ne, float* param, int* ifl, float* photar, float*
 // that they call _sherpa_init_xspec_library before calling any
 // XSPEC routine.
 //
-// Sun's C++ compiler complains if this is declared static
+// Sun's C++ compiler complains if this is declared static.
+//
+// TODO: should we expose this so that anyone who wants to use a user-model
+//       can access it?
+//
 int _sherpa_init_xspec_library()
 {
 
@@ -429,8 +447,13 @@ int _sherpa_init_xspec_library()
     fout.clear();
     fout.close();
 
-    // Try to minimize model chatter for normal operation.
-    FPCHAT( 0 );
+    // We used to set the chatter to 0 but this misses useful info
+    // (like XSPEC can't find the data files) that would be reported
+    // by XSPEC, so use the default XSPEC setting. It does not appear
+    // that this value is read in from ~/.xspec/Xspec.init so set
+    // it here so we have repeatable behavior.
+    //
+    FPCHAT( 10 );
 
     // Set cosmology initial values to XSPEC initial values
     csmph0( 70.0 );
@@ -1400,6 +1423,15 @@ static PyMethodDef XSpecMethods[] = {
   XSPECMODELFCT_C(C_log10con, 1),
 #endif
 
+#ifdef XSPEC_12_12_0
+  XSPECMODELFCT_C_NORM( xsgrbjet, 14 ),  // follow xsgrbcomp and drop the leading c_
+  XSPECMODELFCT_C_NORM( C_vvwDem, 37 ),
+  XSPECMODELFCT_C_NORM( C_vwDem, 21 ),
+  XSPECMODELFCT_C_NORM( C_wDem, 8 ),
+
+  XSPECMODELFCT(zxipab, 5),
+#endif
+
   { NULL, NULL, 0, NULL }
 
 };
