@@ -37,7 +37,7 @@ from sherpa.utils.err import ParameterErr
 #    '(XSConvolutionKernel)'
 # in `xspec/__init__.py`
 #
-XSPEC_MODELS_COUNT = 225
+XSPEC_MODELS_COUNT = 230
 
 # Conversion between wavelength (Angstrom) and energy (keV)
 # The values used are from sherpa/include/constants.hh
@@ -96,7 +96,8 @@ def get_xspec_models():
     # Could just exclude any names that end in 'Model', but this
     # could remove valid model names, so be explicit.
     for n in ['XSModel', 'XSMultiplicativeModel', 'XSAdditiveModel',
-              'XSTableModel', 'XSConvolutionModel', 'XSConvolutionKernel']:
+              'XSTableModel', 'XSConvolutionModel', 'XSConvolutionKernel',
+              'XSBaseParameter', 'XSParameter']:
         remove_item(model_names, n)
 
     # The sirf model - in 12.8.2 and up to 12.9.0d at least - includes
@@ -106,6 +107,12 @@ def get_xspec_models():
     # so it will hopefully be fixed in one of ther 12.9.0 patches.
     remove_item(model_names, 'XSsirf')
 
+    # the grbjet model with XSPEC 12.12.0 (and presumably 12.12.0.a) can
+    # occasionally evaluate to all 0's. This has been reported, but for now
+    # skip this model.
+    #
+    remove_item(model_names, 'XSgrbjet')
+
     models = [getattr(xs, model_name) for model_name in model_names]
     models = list(filter(lambda mod: mod.version_enabled, models))
 
@@ -192,14 +199,11 @@ def assert_is_finite(vals, modelcls, label):
     # mis-match, APEC-style models return all 0 values, so check
     # for this.
     #
-    # Some models seem to return 0's, so skip them for now.
+    # Some models seem to return 0's, so skip them for now:
+    # these models have a default redshift parameter of 0 but
+    # the code complains if z <= 0 and returns 0's.
     #
-    # The *cflow models return 0's because:
-    #     XSVMCF: Require z > 0 for cooling flow models
-    # but the default value is 0 but mkcflox/vmcflow
-    # have a default redshift of 0 in XSPEC 12.10.0 model.dat
-    #
-    if modelcls in [xs.XSmkcflow, xs.XSvmcflow]:
+    if modelcls in [xs.XScph, xs.XSvcph, xs.XSmkcflow, xs.XSvmcflow]:
         # Catch the case when this condition is no longer valid
         #
         assert (vals == 0.0).all(), \
@@ -232,6 +236,28 @@ def test_create_model_instances(clean_astro_ui):
     assert count == XSPEC_MODELS_COUNT
 
 
+@requires_xspec
+def test_check_default_name():
+    import sherpa.astro.xspec as xs
+
+    for clname in dir(xs):
+        if not clname.startswith('XS'):
+            continue
+
+        cls = getattr(xs, clname)
+        if is_proper_subclass(cls, (xs.XSAdditiveModel,
+                                    xs.XSMultiplicativeModel,
+                                    xs.XSConvolutionKernel)):
+
+            # At the moment we have some defaulting to xs... and some just ...
+            # (the forner are convolution cases which should probably be
+            # switched to drop the leading xs).
+            #
+            mdl = cls()
+            expected = clname.lower()
+            assert mdl.name in [expected, expected[2:]]
+
+
 @requires_xspec
 def test_norm_works():
     # Check that the norm parameter for additive models
@@ -405,8 +431,14 @@ def test_xpec_tablemodel_outofbound(clean_astro_ui, make_data_path):
     # when used in the test suite it appears that the tmod
     # global symbol is not created, so need to access the component
     tmod = ui.get_model_component('tmod')
+    elo = numpy.arange(1, 5)
+    ehi = elo + 1
+
+    print(tmod)
+
     with pytest.raises(ParameterErr) as e:
-        tmod.calc([0., .2, 1., 1.], numpy.arange(1, 5), numpy.arange(2, 6))
+        tmod.calc([0., .2, 1., 1.], elo, ehi)
+
     assert 'minimum' in str(e)
 
 
