@@ -1,6 +1,6 @@
 #
 #  Copyright (C) 2008, 2015, 2016, 2017, 2019, 2020, 2021
-#     Smithsonian Astrophysical Observatory
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,46 @@
 #  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
-"""
-Tools for creating, storing, inspecting, and manipulating data sets
+"""Tools for creating, storing, inspecting, and manipulating data sets.
+
+The main classes for representing data sets are `Data1D`,
+`Data1DInt`, and `Data2D`, to handle (x, y), (xlo,
+xhi, y), and (x1, x2, y) data, although there are also
+more-specialized cases, such as `Data1DAsymmetricErrs`. These
+classes build on the `Data` class, which supports dynamic
+filtering of data - to select a subset of the data range - as well as
+data access and model evaluation to match the data range.
+
+The `Filter` class is used to handle data filtering - that
+is, to combine filters such as selecting the range a to b (``notice``)
+and hiding the range c to d (``ignore``). This is used with the
+`DataSpace1D` and `DataSpace2D` classes to handle
+evaluating models on different grids to the data, and then converting
+back to the data space, whether by rebinnig or interpolation.
+
+Notebook support
+----------------
+
+The Data objects support the rich display protocol of IPython, with
+HTML display of a table of information highlighting the relevant data.
+Examples can be found at [NoteBook]_.
+
+References
+----------
+
+.. [NoteBook] https://sherpa.readthedocs.io/en/latest/NotebookSupport.html
+
+Examples
+--------
+
+Create a data set representing the independent axis (``x``) and
+dependent axis (``y``) then filter to select only those values between
+500-520 and 530-700:
+
+>>> d1 = Data1D('example', x, y)
+>>> d1.notice(500, 700)
+>>> d1.ignore(520, 530)
+
 """
 import warnings
 from abc import ABCMeta
@@ -29,7 +67,7 @@ import numpy
 from sherpa.models.regrid import EvaluationSpace1D
 from sherpa.utils.err import DataErr
 from sherpa.utils import SherpaFloat, NoNewAttributesAfterInit, \
-    print_fields, create_expr, calc_total_error, bool_cast, \
+    print_fields, create_expr, create_expr_integrated, calc_total_error, bool_cast, \
     filter_bins, parallel_map_funcs
 from sherpa.utils import formatting
 
@@ -396,8 +434,7 @@ class Filter():
 
     @property
     def mask(self):
-        """
-        Mask array for dependent variable
+        """Mask array for dependent variable
 
         Returns
         -------
@@ -422,8 +459,7 @@ class Filter():
             self._mask = numpy.asarray(val, numpy.bool_)
 
     def apply(self, array):
-        """
-        Apply this filter to an array
+        """Apply this filter to an array
 
         Parameters
         ----------
@@ -433,31 +469,124 @@ class Filter():
         Returns
         -------
         array_like : filtered array
+
+        Raises
+        ------
+        sherpa.utils.err.DataErr
+            The filter has removed all elements or there is a
+            mis-match between the `mask` and the ``array`` argument.
+
+        See Also
+        --------
+        notice
+
         """
         if array is None:
             return
 
+        # Note that mask may not be a boolean but an array.
         if self.mask is False:
             raise DataErr('notmask')
 
-        if self.mask is not True:  # mask is not False and not True, so it's something else we'll try to use as an array
-            array = numpy.asarray(array)
-            if array.shape != self.mask.shape:
-                raise DataErr('mismatch', 'mask', 'data array')
-            return array[self.mask]
+        if self.mask is True:
+            return array
 
-        return array
+        array = numpy.asarray(array)
+        if array.shape != self.mask.shape:
+            raise DataErr('mismatch', 'mask', 'data array')
+        return array[self.mask]
 
-    def notice(self, mins, maxes, axislist, ignore=False):
+    def notice(self, mins, maxes, axislist, ignore=False, integrated=False):
+        """Select a range to notice or ignore (remove).
+
+        The ``axislist`` argument is expected to be sent the
+        independent axis of a `Data` object - so ``(x, )`` for
+        one-dimensional data, ``(xlo, xhi)`` for integrated
+        one-dimensional data, ``(x0, x1)`` for two-dimensional data,
+        and ``(x0lo, x1lo, x0hi, x1hi)`` for integrated two-dimensinal
+        data. The ``mins`` and ``maxes`` must then be set to match
+        this ordering.
+
+        Parameters
+        ----------
+        mins : sequence of values
+           The minimum value of the valid range (elements may be None
+           to indicate no lower bound). When not None, it is treated
+           as an inclusive limit, so points >= min are included.
+        maxes : sequence of values
+           The maximum value of the valid range (elements may be None
+           to indicate no upper bound). It is treated as an inclusive
+           limit (points <= max) when integrated is False, and an
+           exclusive limit (points < max) when integrated is True.
+        axislist: sequence of arrays
+           The axis to apply the range to. There must be the same
+           number of elements in mins, maxes, and axislist.  The
+           number of elements of each element of axislist must also
+           agree (the cell values do not need to match).
+        ignore : bool, optional
+           If True the range is to be ignored, otherwise it is
+           included.  The default is to include the range.
+        integrated : bool, optional
+           Is the data integrated (we have low and high bin edges)?  The
+           default is False. When True it is expected that axislist
+           contains a even number of rows, where the odd values are the
+           low edges and the even values the upper edges, and that the
+           mins and maxes only ever contain a single value, given in
+           (None, hi) and (lo, None) ordering.
+
+        See Also
+        --------
+        apply
+
+        Examples
+        --------
+
+        Select points in xs which are in the range 1.5 <= x <= 4:
+
+        >>> f = Filter()
+        >>> f.mask
+        True
+        >>> xs = [1, 2, 3, 4, 5]
+        >>> f.notice([1.5], [4], (xs, ))
+        >>> f.mask
+        array([False,  True,  True,  True, False])
+
+        Filter the data to select all points with x0 >= 1.5 and x1 <= 4:
+
+        >>> f = Filter()
+        >>> x0 = [1, 1.4, 1.6, 2, 3]
+        >>> x1 = [2, 2, 4, 4, 6]
+        >>> f.notice([1.5, None], [None, 4], (x0, x1))
+        >>> f.mask
+        array([False, False,  True,  True, False])
+
+        For integrated data sets the lower and upper edges should be
+        sent separately with the max and min limits, along with
+        setting the integrated flag. The following selects the bins
+        that cover the range 2 to 4 and 1.5 to 3.5:
+
+        >>> xlo = [1, 2, 3, 4, 5]
+        >>> xhi = [2, 3, 4, 5, 6]
+        >>> f = Filter()
+        >>> f.notice([None, 2], [4, None], (xlo, xhi), integrated=True)
+        >>> f.mask
+        array([False,  True,  True,  False, False])
+        >>> f.notice([None, 1.5], [3.5, None], (xlo, xhi), integrated=True)
+        >>> f.mask
+        array([True,  True,  True,  False, False])
+
+        """
+
+        # If integrated is True then we should have an even number
+        # of axislist elements, but we do not require this.
+        #
         ignore = bool_cast(ignore)
-        if str in [type(min) for min in mins]:
-            raise DataErr('typecheck', 'lower bound')
-        elif str in [type(max) for max in maxes]:
-            raise DataErr('typecheck', 'upper bound')
-        elif str in [type(axis) for axis in axislist]:
-            raise DataErr('typecheck', 'grid')
+        for vals, label in zip([mins, maxes, axislist],
+                               ['lower bound', 'upper bound', 'grid']):
+            if any([isinstance(val, str) for val in vals]):
+                raise DataErr('typecheck', label)
 
-        mask = filter_bins(mins, maxes, axislist)
+        mask = filter_bins(mins, maxes, axislist, integrated=integrated)
 
         if mask is None:
             self.mask = not ignore
@@ -577,6 +706,7 @@ class Data(NoNewAttributesAfterInit, BaseData):
         Return the dimensions of this data space as a tuple of tuples.
         The first element in the tuple is a tuple with the dimensions of the data space, while the second element
         provides the size of the dependent array.
+
         Returns
         -------
         tuple
@@ -588,6 +718,7 @@ class Data(NoNewAttributesAfterInit, BaseData):
     def indep(self):
         """
         Return the grid of the data space associated with this data set.
+
         Returns
         -------
         tuple of array_like
@@ -747,7 +878,7 @@ class Data(NoNewAttributesAfterInit, BaseData):
 
         Returns
         -------
-        axis : array or `None`
+        axis : array or None
            The systematic error for each data point. A value of
            `None` is returned if the data set has no systematic
            errors.
@@ -826,8 +957,9 @@ class Data(NoNewAttributesAfterInit, BaseData):
     def apply_filter(self, data):
         return self._data_space.filter.apply(data)
 
-    def notice(self, mins, maxes, ignore=False):
-        self._data_space.filter.notice(mins, maxes, self.get_indep(), ignore)
+    def notice(self, mins, maxes, ignore=False, integrated=False):
+        self._data_space.filter.notice(mins, maxes, self.get_indep(),
+                                       ignore=ignore, integrated=integrated)
 
     def ignore(self, *args, **kwargs):
         kwargs['ignore'] = True
@@ -881,7 +1013,7 @@ class DataSimulFit(NoNewAttributesAfterInit):
         The name for the collection of data.
     datasets : sequence of Data objects
         The datasets to be stored; there must be at least one. They are
-        assumed to behave as sherpa.data.Data objects, but there is no
+        assumed to behave as `sherpa.data.Data` objects, but there is no
         check for this condition.
 
     Attributes
@@ -1079,15 +1211,72 @@ class Data1D(Data):
         return err
 
     def get_filter(self, format='%.4f', delim=':'):
-        # for derived intergrated classes, this will return values in center of
-        # bin.
+        """Return the data filter as a string.
+
+        Parameters
+        ----------
+        format : str, optional
+            The formatting of the numeric values.
+        delim : str, optional
+            The string used to mark the low-to-high range.
+
+        Returns
+        -------
+        filter : str
+           The filter, represented as a collection of single values or
+           ranges, separated by commas.
+
+        See Also
+        --------
+        get_filter_expr, ignore, notice
+
+        Examples
+        --------
+
+        >>> x = np.asarray([1, 2, 3, 5, 6])
+        >>> y = np.ones(5)
+        >>> d = Data1D('example', x, y)
+        >>> d.get_filter()
+        '1.0000:6.0000'
+        >>> d.ignore(2.5, 4.5)
+        >>> d.get_filter()
+        '1.0000:2.0000,5.0000:6.0000'
+
+        >>> d.get_filter(format='%i', delim='-')
+        '1-2,5-6'
+
+        """
+
         x = self.get_x(filter=True)
-        mask = numpy.ones(len(x), dtype=bool)
         if numpy.iterable(self.mask):
             mask = self.mask
-        return create_expr(x, mask, format, delim)
+        else:
+            mask = numpy.ones(len(x), dtype=bool)
+        return create_expr(x, mask=mask, format=format, delim=delim)
 
     def get_filter_expr(self):
+        """Return the data filter as a string along with the units.
+
+        This is a specialised version of get_filter which adds the
+        axis units.
+
+        Returns
+        -------
+        filter : str
+           The filter, represented as a collection of single values or
+           ranges, separated by commas.
+
+        See Also
+        --------
+        get_filter
+
+        Examples
+        --------
+
+        >>> d.get_filter_expr()
+        '1.0000-2.0000,5.0000-6.0000 x'
+
+        """
         return self.get_filter(delim='-') + ' ' + self.get_xlabel()
 
     def to_plot(self, yfunc=None, staterrfunc=None):
@@ -1118,6 +1307,51 @@ class Data1D(Data):
             return data_space.grid
 
     def notice(self, xlo=None, xhi=None, ignore=False):
+        """Notice or ignore the given range.
+
+        Ranges are inclusive for both the lower and upper limits.
+
+        Parameters
+        ----------
+        xlo, xhi : number or None, optional
+            The range to change. A value of None means the minimum or
+            maximum permitted value.
+        ignore : bool, optional
+            Set to True if the range should be ignored. The default is
+            to notice the range.
+
+        See Also
+        --------
+        get_filter, get_filter_expr
+
+        Notes
+        -----
+        If no ranges have been ignored then a call to `notice` with
+        `ignore=False` will select just the `lo` to `hi` range, and
+        exclude any bins outside this range. If there has been a
+        filter applied then the range `lo` to `hi` will be added to
+        the range of noticed data (when `ignore=False`).
+
+        Examples
+        --------
+
+        >>> x = np.arange(0.4, 2.6, 0.2)
+        >>> y = np.ones_like(x)
+        >>> d = Data1D('example', x, y)
+        >>> d.x[0], d.x[-1]
+        (0.4, 2.4000000000000004)
+        >>> d.notice()
+        >>> d.get_filter(format='%.1f')
+        '0.4:2.4'
+        >>> d.notice(0.8, 1.2)
+        >>> d.get_filter(format='%.1f')
+        '0.8:1.2'
+        >>> d.notice(1.5, 2.1)
+        >>> d.get_filter(format='%.1f')
+        '0.8:1.2,1.6:2.0'
+
+        """
+
         Data.notice(self, (xlo,), (xhi,), ignore)
 
     @property
@@ -1130,7 +1364,9 @@ class Data1D(Data):
 
 class Data1DAsymmetricErrs(Data1D):
     """1-D data set with asymmetric errors
-    Note: elo and ehi shall be stored as delta values from y"""
+
+    Note: elo and ehi shall be stored as delta values from y
+    """
 
     _fields = ("name", "x", "y", "staterror", "syserror", "elo", "ehi")
 
@@ -1162,14 +1398,142 @@ class Data1DInt(Data1D):
 
     def get_x(self, filter=False, model=None, use_evaluation_space=False):
         indep = self.get_evaluation_indep(filter, model, use_evaluation_space)
+        if len(indep) == 1:
+            # assume all data has been filtered out
+            return numpy.asarray([])
+
         return (indep[0] + indep[1]) / 2.0
 
     def get_xerr(self, filter=False, model=None):
-        xlo, xhi = self.get_evaluation_indep(filter, model)
+        indep = self.get_evaluation_indep(filter, model)
+        if len(indep) == 1:
+            # assume all data has been filtered out
+            return numpy.asarray([])
+
+        xlo, xhi = indep
         return xhi - xlo
 
+    def get_filter(self, format='%.4f', delim=':'):
+        """Return the data filter as a string.
+
+        For each noticed range the filter is reported as
+        starting at the low edge of the first bin and ends
+        at the upper edge of the last bin in the range.
+
+        .. versionchanged:: 4.14.0
+           Prior to 4.14.0 the filter used the mid-point of the bin,
+           not its low or high value.
+
+        Parameters
+        ----------
+        format : str, optional
+            The formatting of the numeric values.
+        delim : str, optional
+            The string used to mark the low-to-high range.
+
+        Returns
+        -------
+        filter : str
+           The filter, represented as a collection of ranges separated
+           by commas.
+
+        See Also
+        --------
+        get_filter_expr, ignore, notice
+
+        Examples
+        --------
+
+        >>> xlo = np.asarray([1, 2, 3, 5, 6])
+        >>> xhi = xlo + 1
+        >>> y = np.ones(5)
+        >>> d = Data1DInt('example', xlo, xhi, y)
+        >>> d.get_filter()
+        '1.0000:7.0000'
+        >>> d.ignore(2.5, 4.5)
+        >>> d.get_filter()
+        '1.0000:2.0000,5.0000:7.0000'
+
+        >>> d.get_filter(format='%i', delim='-')
+        '1-2,5-7'
+
+        """
+
+        # We could just use the middle of each bin (which is what
+        # would happen if we didn't over-ride Data1D.filter) but let's
+        # use the start and end values for each selected bin.
+        #
+        indep = self.get_evaluation_indep(filter=True)
+        if len(indep) == 1:
+            # assume all data has been filtered out
+            return ''
+
+        if numpy.iterable(self.mask):
+            mask = self.mask
+        else:
+            # Unlike create_expr we do not need to send
+            # a mask in here
+            mask = None
+
+        return create_expr_integrated(indep[0], indep[1], mask=mask,
+                                      format=format, delim=delim)
+
     def notice(self, xlo=None, xhi=None, ignore=False):
-        Data.notice(self, (None, xlo), (xhi, None), ignore)
+        """Notice or ignore the given range.
+
+        Ranges are inclusive for the lower limit and exclusive
+        for the upper limit.
+
+        .. versionchanged:: 4.14.0
+           Filtering Data1DInt datasets has been improved to fix a
+           number of corner cases. As part of this the upper limit has
+           been changed to be exclusive whereas previously it was not
+           obvious what the filter was doing.
+
+        Parameters
+        ----------
+        xlo, xhi : number or None, optional
+            The range to change. A value of None means the minimum or
+            maximum permitted value.
+        ignore : bool, optional
+            Set to True if the range should be ignored. The default is
+            to notice the range.
+
+        See Also
+        --------
+        get_filter, get_filter_expr
+
+        Notes
+        -----
+        If no ranges have been ignored then a call to `notice` with
+        `ignore=False` will select just the `lo` to `hi` range, and
+        exclude any bins outside this range. If there has been a
+        filter applied then the range `lo` to `hi` will be added to
+        the range of noticed data (when `ignore=False`).
+
+        Examples
+        --------
+
+        >>> edges = np.arange(0.4, 2.6, 0.2)
+        >>> xlo, xhi = edges[:-1], edges[1:]
+        >>> y = np.ones_like(xlo)
+        >>> d = Data1DInt('example', xlo, xhi, y)
+        >>> d.xlo[0], d.xhi[-1]
+        (0.4, 2.400000000000001)
+        >>> d.get_filter(format='%.1f')
+        '0.4:2.4'
+        >>> d.notice(0.8, 1.9)
+        >>> d.get_filter(format='%.1f')
+        '0.8:2.0'
+
+        >>> d.notice()
+        >>> d.get_filter(format='%.1f')
+        '0.4:2.4'
+
+        """
+
+        Data.notice(self, (None, xlo), (xhi, None),
+                    ignore=ignore, integrated=True)
 
     @property
     def xlo(self):
@@ -1300,7 +1664,8 @@ class Data2D(Data):
         return self.get_x1()
 
     def notice(self, x0lo=None, x0hi=None, x1lo=None, x1hi=None, ignore=False):
-        Data.notice(self, (x0lo, x1lo), (x0hi, x1hi), ignore)
+        Data.notice(self, (x0lo, x1lo), (x0hi, x1hi),
+                    ignore=ignore)
 
 
 class Data2DInt(Data2D):
@@ -1325,7 +1690,8 @@ class Data2DInt(Data2D):
         return (indep.x1lo + indep.x1hi) / 2.0
 
     def notice(self, x0lo=None, x0hi=None, x1lo=None, x1hi=None, ignore=False):
-        Data.notice(self, (None, None, x0lo, x1lo), (x0hi, x1hi, None, None), ignore)
+        Data.notice(self, (None, None, x0lo, x1lo), (x0hi, x1hi, None, None),
+                    ignore=ignore, integrated=True)
 
 
 # Notebook representations
@@ -1334,7 +1700,6 @@ def html_data1d(data):
     """HTML representation: Data1D
 
     If have matplotlib then plot the data, otherwise summarize it.
-
     """
 
     from sherpa.plot import DataPlot, backend
@@ -1389,7 +1754,6 @@ def html_data1dint(data):
     """HTML representation: Data1DInt
 
     If have matplotlib then plot the data, otherwise summarize it.
-
     """
 
     from sherpa.plot import DataHistogramPlot, backend
