@@ -24,8 +24,10 @@ import signal
 
 from functools import wraps
 
+import numpy as np
+
 from numpy import arange, array, abs, iterable, sqrt, where, \
-    ones_like, isnan, isinf, float, float32, finfo, any, int
+    ones_like, isnan, isinf, any
 from sherpa.utils import NoNewAttributesAfterInit, print_fields, erf, \
     bool_cast, is_in, is_iterable, list_to_open_interval, sao_fcmp
 from sherpa.utils.err import FitErr, EstErr, SherpaErr
@@ -68,7 +70,7 @@ class StatInfoResults(NoNewAttributesAfterInit):
     ids : sequence of int or str
        The data set ids (it may be a tuple or array) included in the
        results.
-    bkg_ids: sequence of int or str, or `None`
+    bkg_ids: sequence of int or str, or None
        The background data set ids (it may be a tuple or array)
        included in the results, if any.
     statname : str
@@ -80,16 +82,15 @@ class StatInfoResults(NoNewAttributesAfterInit):
     dof: int
        The number of degrees of freedom in the fit (the number of
        bins minus the number of free parameters).
-    qval: number or `None`
+    qval: number or None
        The Q-value (probability) that one would observe the reduced
        statistic value, or a larger value, if the assumed model is
        true and the current model parameters are the true parameter
        values. This will be `None` if the value can not be calculated
        with the current statistic (e.g. the Cash statistic).
-    rstat: number of `None`
+    rstat: number or None
        The reduced statistic value (the `statval` field divided by
        `dof`). This is not calculated for all statistics.
-
     """
 
     # The fields to include in the __str__ output.
@@ -156,8 +157,8 @@ def _cleanup_chi2_name(stat, data):
 
     Parameters
     ----------
-    stat : a sherpa.stats.Stat instance
-    data : a sherpa.data.Data or sherpa.data.DataSimulFit instance
+    stat : `sherpa.stats.Stat`
+    data : `sherpa.data.Data` or `sherpa.data.DataSimulFit`
 
     Returns
     -------
@@ -195,9 +196,9 @@ class FitResults(NoNewAttributesAfterInit):
 
     Attributes
     ----------
-    datasets : sequence of int or str, or `None`
+    datasets : sequence of int or str, or None
        A sequence of the data set ids included in the results.
-    itermethodname : str or `None`
+    itermethodname : str or None
        What iterated-fit scheme was used, if any.
     methodname : str
        The name of the optimisation method used (in lower case).
@@ -215,19 +216,19 @@ class FitResults(NoNewAttributesAfterInit):
     istatval : number
        The statistic value at the start of the fit.
     dstatval : number
-       The change in the statistic value (`istatval - statval`).
+       The change in the statistic value (``istatval - statval``).
     numpoints : int
        The number of bins used in the fits.
     dof : int
        The number of degrees of freedom in the fit (the number of
        bins minus the number of free parameters).
-    qval : number or `None`
+    qval : number or None
        The Q-value (probability) that one would observe the reduced
        statistic value, or a larger value, if the assumed model is
        true and the current model parameters are the true parameter
        values. This will be `None` if the value can not be calculated
-       with the current statistic (e.g.  the Cash statistic).
-    rstat : number or `None`
+       with the current statistic (e.g. the Cash statistic).
+    rstat : number or None
        The reduced statistic value (the `statval` field divided by
        `dof`). This is not calculated for all statistics.
     message : str
@@ -238,10 +239,10 @@ class FitResults(NoNewAttributesAfterInit):
        The number of model evaluations made during the fit.
     extra_output
        The ``extra_output`` field from the fit.
-    covar : tuple or `None`
+    covar : tuple or None
        The covariance matrix from the best-fit location, if provided
        by the optimiser.
-    modelvals : NumPy array
+    modelvals : `array`
        The values of the best-fit model evaluated for the data.
 
     """
@@ -389,7 +390,6 @@ class ErrorEstResults(NoNewAttributesAfterInit):
        The number of fits performed during the error analysis.
     extra_output
        The ``extra_output`` field from the fit.
-
     """
 
     # The fields to include in the __str__ output.
@@ -635,7 +635,7 @@ class IterFit(NoNewAttributesAfterInit):
 
         Raises
         ------
-        sherpa.utils.err.FitErr
+        `sherpa.utils.err.FitErr`
             This exception is raised if the statistic is not supported.
             This method can only be used with Chi-Square statistics
             with errors.
@@ -722,7 +722,7 @@ class IterFit(NoNewAttributesAfterInit):
         # Keep record of current and previous statistics;
         # when these are within some tolerace, Primini's method
         # is done.
-        previous_stat = float32(finfo(float32).max)
+        previous_stat = np.finfo(np.float32).max
         current_stat = statfunc(pars)[0]
         nfev = 0
         iters = 0
@@ -773,9 +773,9 @@ class IterFit(NoNewAttributesAfterInit):
                  statkwargs=None, cache=True):
         """Exclude points that are significately far away from the best fit.
 
-        The `sigmarej` scheme is based on the IRAF `sfit` function
+        The `sigmarej` scheme is based on the IRAF ``sfit`` function
         [3]_, where after a fit data points are excluded if the value
-        of `(data-model) / error` exceeds a threshold, and the data
+        of ``(data-model) / error`` exceeds a threshold, and the data
         re-fit. This removal of data points continues until the fit
         has converged or a maximum number of iterations has been reached.
         The error removal can be asymmetric, since there are separate
@@ -783,7 +783,7 @@ class IterFit(NoNewAttributesAfterInit):
 
         Raises
         ------
-        sherpa.utils.err.FitErr
+        `sherpa.utils.err.FitErr`
             This exception is raised if the statistic is not
             supported. This method can only be used with
             Chi-Square statistics with errors.
@@ -998,23 +998,23 @@ class Fit(NoNewAttributesAfterInit):
 
     Parameters
     ----------
-    data : sherpa.data.Data or sherpa.data.DataSimulFit instance
+    data : `sherpa.data.Data` or `sherpa.data.DataSimulFit`
        The data to be fit.
-    model : sherpa.models.model.Model or sherpa.models.model.SimulFitModel instance
-       The model to fit to the data. It should match the `data` parameter
-       (i.e. be a SimulFitModel object when data is a DataSimulFit).
-    stat : sherpa.stats.Stat instance or None, optional
+    model : `sherpa.models.model.Model` or `sherpa.models.model.SimulFitModel`
+       The model to fit to the data. It should match the ``data`` parameter
+       (i.e. be a `SimulFitModel` object when data is a `DataSimulFit`).
+    stat : `sherpa.stats.Stat` or `None`, optional
        The statistic object to use. If not given then
-       :py:class:`~sherpa.stats.Chi2Gehrels` is used.
-    method : sherpa.optmethods.OptMethod instance or None, optional
-       The optimiser to use. If not given then
-       :py:class:`~sherpa.optmethods.LevMar` is used.
-    estmethod : sherpa.estmethod.EstMethod instance or None, optional
+       `Chi2Gehrels` is used.
+    method : `sherpa.optmethods.OptMethod` instance or None, optional
+       The optimiser to use. If not given then `sherpa.optmethods.LevMar`
+       is used.
+    estmethod : `sherpa.estmethods.EstMethod` or None, optional
        The class used to calculate errors. If not given then
-       :py:class:`~sherpa.estmethods.Covariance` is used.
+       `sherpa.estmethods.Covariance` is used.
     itermethod_opts : dict or None, optional
        If set, defines the iterated-fit method and options to use.
-       It is passed through to :py:class:`~sherpa.fit.IterFit`.
+       It is passed through to `IterFit`.
 
     """
 
@@ -1083,7 +1083,7 @@ class Fit(NoNewAttributesAfterInit):
     def guess(self, **kwargs):
         """Guess parameter values and limits.
 
-        The model's :py:meth:`~sherpa.models.model.Model.guess` method
+        The model's `sherpa.models.model.Model.guess` method
         is called with the data values (the dependent axis of the
         data set) and the ``kwargs`` arguments.
         """
@@ -1130,16 +1130,15 @@ class Fit(NoNewAttributesAfterInit):
 
         Returns
         -------
-        chisq : array or `None`
+        chisq : array or None
            The chi-square value for each bin of the data, using the
-           current statistic (as set by `set_stat`).  A value of
-           `None` is returned if the statistic is not a chi-square
+           current statistic.
+           A value of `None` is returned if the statistic is not a chi-square
            distribution.
 
         See Also
         --------
         calc_stat, calc_stat_info
-
         """
 
         # Since there is some setup work needed before calling
@@ -1159,7 +1158,7 @@ class Fit(NoNewAttributesAfterInit):
 
         Returns
         -------
-        statinfo : StatInfoResults instance
+        statinfo : `StatInfoResults` instance
            The current statistic value.
 
         See Also
@@ -1189,7 +1188,7 @@ class Fit(NoNewAttributesAfterInit):
 
         Parameters
         ----------
-        outfile : str or `None`, optional
+        outfile : str or None, optional
            If not `None` then information on the fit is written to
            this file.
         clobber : bool, optional
@@ -1199,12 +1198,12 @@ class Fit(NoNewAttributesAfterInit):
 
         Returns
         -------
-        fitres : FitResults instance
+        fitres : `FitResults`
 
         Raises
         ------
-        sherpa.utils.err.FitErr
-           This is raised if `clobber` is `False` and `outfile` already
+        `sherpa.utils.err.FitErr`
+           This is raised if ``clobber`` is ``False`` and ``outfile`` already
            exists or if all the bins have been masked out of the fit.
 
         See Also
@@ -1213,7 +1212,7 @@ class Fit(NoNewAttributesAfterInit):
 
         Notes
         -----
-        The file created when `outfile` is set is a simple ASCII
+        The file created when ``outfile`` is set is a simple ASCII
         file with a header line containing the text
         "# nfev statistic" and then a list of the thawed parameters,
         and then one line for each iteration, with the values separated
@@ -1254,7 +1253,7 @@ class Fit(NoNewAttributesAfterInit):
 
         # check if any parameter values are at boundaries,
         # and warn user.
-        tol = finfo(float32).eps
+        tol = np.finfo(np.float32).eps
         param_warnings = ""
         for par in self.model.pars:
             if not par.frozen:
@@ -1289,13 +1288,13 @@ class Fit(NoNewAttributesAfterInit):
 
         Parameters
         ----------
-        *others : sherpa.fit.Fit instances
+        *others : `sherpa.fit.Fit` instances
             The ``data`` and ``model`` attributes of these arguments
             are used, along with those from the object.
 
         Returns
         -------
-        fitres : FitResults instance
+        fitres : `FitResults`
 
         See Also
         --------
@@ -1321,26 +1320,26 @@ class Fit(NoNewAttributesAfterInit):
 
         Parameters
         ----------
-        methoddict : dict or `None`, optional
+        methoddict : dict or None, optional
             A dictionary mapping from lower-cased method name to
             the associated optimisation method instance to use. This
             is only used if the method is changed, as described in
             the Notes section below.
-        parlist : seqquence of sherpa.model.parameter.Parameter instances or `None`, optional
+        parlist : sequence of `sherpa.models.parameter.Parameter` or None, optional
             The names of the parameters for which the errors should
             be calculated. If set to `None` then all the thawed
             parameters are used.
 
         Returns
         -------
-        res : ErrorEstResults instance
+        res : ErrorEstResults
 
         Raises
         ------
-        sherpa.utils.err.EstErr
+        `sherpa.utils.err.EstErr`
            If any parameter in parlist is not valid (i.e. is not
            thawed or is not a member of the model expression being
-           fit), or if the statistic is :py:class:`~sherpa.stats.LeastSq`,
+           fit), or if the statistic is `~sherpa.stats.LeastSq`,
            or if the reduced chi-square value of the current parameter
            values is larger than the ``max_rstat`` option (for
            chi-square statistics).
@@ -1356,10 +1355,10 @@ class Fit(NoNewAttributesAfterInit):
         new best-fit location. This can repeat until the ``maxfits``
         option is reached.
 
-        Unless the :py:class:`~sherpa.estmethods.Covariance` estimator
-        is being used, ot the ``fast`` option is unset, then the method
-        will be changed to :py:class:`~sherpa.optmethods.NelderMead` (for
-        likelihood-based statistics) or :py:class:`~sherpa.optmethods.LevMar`
+        Unless the `~sherpa.estmethods.Covariance` estimator
+        is being used, or the ``fast`` option is unset, then the method
+        will be changed to `~sherpa.optmethods.NelderMead` (for
+        likelihood-based statistics) or `~sherpa.optmethods.LevMar`
         (for chi-square based statistics) whilst calculating the
         errors.
         """
