@@ -1,6 +1,6 @@
 #
 #  Copyright (C) 2011, 2015, 2016, 2018, 2019, 2020, 2021
-#      Smithsonian Astrophysical Observatory
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -20,12 +20,8 @@
 
 """Monte-Carlo Markov Chain support for low-count data (Poisson statistics).
 
-The ``sherpa.sim`` module provides support for exploring the posterior
-probability density of parameters in a fit to low-count data, for
-which Poisson statistics hold, using a Bayesian algorithm and a
-Monte-Carlo Markov Chain (MCMC). It was originally known as the
-pyBLoCXS (python Bayesian Low-Count X-ray Spectral) package [1]_, but
-has since been incorporated into Sherpa.
+The ``sherpa.sim`` module provides a Bayesian algorithm with
+Monte-Carlo Markov Chain (MCMC) for Poisson data.
 
 The Sherpa UI modules - e.g. `sherpa.ui` and `sherpa.astro.ui` - provide
 many of the routines described below (e.g. ``list_samplers``).
@@ -219,7 +215,7 @@ from sherpa.optmethods import LevMar
 info = logging.getLogger("sherpa").info
 _log = logging.getLogger("sherpa")
 
-_tol = numpy.finfo(numpy.float).eps
+_tol = numpy.finfo(float).eps
 
 string_types = (str, )
 
