@@ -1,5 +1,6 @@
 #
-#  Copyright (C) 2017, 2018, 2020  Smithsonian Astrophysical Observatory
+#  Copyright (C) 2017, 2018, 2020, 2021
+#  Smithsonian Astrophysical Observatory
 #
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -24,10 +25,12 @@ Note that this test is almost duplicated in
 sherpa/astro/ui/tests/test_astro_ui_unit.py
 """
 
+import numpy as np
+
 import pytest
 
 from sherpa import ui
-from sherpa.utils.err import ArgumentTypeErr
+from sherpa.utils.err import ArgumentTypeErr, IdentifierErr
 
 
 # This is part of #397
@@ -69,10 +72,53 @@ def test_all_has_no_repeated_elements():
 
 
 @pytest.mark.parametrize("func", [ui.notice_id, ui.ignore_id])
-def test_check_ids_not_none(func):
-    """Check they error out when id is None"""
+@pytest.mark.parametrize("lo", [None, 1, "1:5"])
+def test_check_ids_not_none(func, lo):
+    """Check they error out when id is None
+
+    There used to be the potential for different behavior depending
+    if the lo argument was a string or not,hence the check.
+    """
 
     with pytest.raises(ArgumentTypeErr) as exc:
-        func(None)
+        func(None, lo)
 
     assert str(exc.value) == "'ids' must be an identifier or list of identifiers"
+
+
+@pytest.mark.parametrize("func", [ui.notice, ui.ignore])
+@pytest.mark.parametrize("lo,hi", [(1, 5), (1, None), (None, 5), (None, None),
+                                   ("1:5", None)])
+def test_filter_no_data_is_an_error(func, lo, hi, clean_ui):
+    """Does applying a filter lead to an error?
+
+    This test was added because it was noted that an update for Python 3
+    had lead to an error state not being reached.
+    """
+
+    with pytest.raises(IdentifierErr) as ie:
+        func(lo, hi)
+
+    assert str(ie.value) == 'No data sets found'
+
+
+def test_save_filter_data1d(tmp_path, clean_ui):
+    """Check save_filter [Data1D]"""
+
+    x = np.arange(1, 11, dtype=np.int16)
+    ui.load_arrays(1, x, x)
+
+    ui.notice(2, 4)
+    ui.notice(6, 8)
+
+    outfile = tmp_path / "filter.dat"
+    ui.save_filter(str(outfile))
+
+    expected = [0, 1, 1, 1, 0, 1, 1, 1, 0, 0]
+
+    d = ui.unpack_data(str(outfile), colkeys=['X', 'FILTER'])
+    assert isinstance(d, ui.Data1D)
+    assert d.x == pytest.approx(x)
+    assert d.y == pytest.approx(expected)
+    assert d.staterror is None
+    assert d.syserror is None
